<?php

# A web service to process files uploaded by a user.
# Requires authentication through the auth.php web service.
# Input: (none).
# Output: number of file processed, 0 if none.
# false if you are not logged in,
# -1 at once if there is already another processing in progress at the moment.

# When this script is called from the command line by the schedule.php web
# service a cookie value must be passed as a argument, since there is no
# browser passing it. We can set this value directly into the $_COOKIE array
# and we will be authenticated. E.g.:
# # php process.php '1gissjmehf4mlag30gpkuga6l3'
if (isset($argc)) {
  if ($argc != 2) {
    exit("syntax: $argv[0] COOKIE");
  }
  require '../lib/config.php';
  $_COOKIE[\CsvUpload\Config\SESSION_NAME] = $argv[1];
}

require '../controller/auth.php';
require '../controller/upload.php';
require '../model/aps.php';
require '../lib/service.php';

# Check authentication!
if (! \CsvUpload\Auth\logged_in()) {
  json_exit();
}

# Detect whether there is already a processing in progress
# acquiring a write lock to make absolutely sure that only
# one processing can be run at the same time!
$fp = fopen($lock_file = \CsvUpload\Upload\lock_file(), 'w');

# LOCK_EX = exclusive lock (writer)
# LOCK_NB = non blocking locking operation
if (!flock($fp, LOCK_EX | LOCK_NB)) {
  json_export(-1);
}

# It is very important that this script, that may run for a long time, closes
# the PHP session ASAP. This is to allow other scripts like state.php to run
# concurrently. Note that although we close the session, the $_SESSION array
# is still available till the end of the request: this is important since we
# make some calls that depend on it.
# See:
# http://php.net/manual/en/ref.session.php#64525
# http://www.php.net/session_write_close
session_write_close();

# Update processing progress state.
# File locking is needed to make reads for other processes consistent.
function update_status($file, $state) {
  global $states;

  $states[$file] = $state;
  file_put_contents(\CsvUpload\upload\progress_file(),
    json_encode($states), LOCK_EX);
}

$files = \CsvUpload\upload\ls(TRUE);
$states = array_fill_keys(array_map('basename', $files), 'todo');
foreach ($files as $path) {
  $file = basename($path);
  update_status($file, 'processing');
  # DEBUG
  #sleep(20);
  update_status($file, \CsvUpload\Database\load_csv($path) ? 'done' : 'error');

  # Remove every file as soon as it has been processed.
  unlink($path);
}
# Remove user directory, e.g. csvupload/3 since it is now empty.
if (count($files)) {
  rmdir(\CsvUpload\Upload\home_dir());
}

flock($fp, LOCK_UN);  # Release the lock.
fclose($fp);
unlink($lock_file);

json_export(count($files));

# vim: set filetype=php expandtab tabstop=2 shiftwidth=2 autoindent smartindent:
?>
